package signald

import (
	"encoding/json"
	"log"
	"net"
)

// Signald is a connection to a signald instance.
type Signald struct {
	socket     net.Conn
	SocketPath string
}

func crash(err error) {
	if err != nil {
		panic(err)
	}
}

// Connect connects to the signad socket
func (s *Signald) Connect() {
	if s.SocketPath == "" {
		s.SocketPath = "/var/run/signald/signald.sock"
	}
	socket, err := net.Dial("unix", s.SocketPath)
	crash(err)
	s.socket = socket
	log.Print("Connected to signald socket ", socket.RemoteAddr().String())
}

// Listen listens for events from signald
func (s *Signald) Listen(c chan interface{}) {
	// we create a decoder that reads directly from the socket
	d := json.NewDecoder(s.socket)

	var msg interface{}

	for {
		crash(d.Decode(&msg))
		log.Print(msg)
		c <- msg
	}
}

// Send sends a request to signald.
func (s *Signald) Send(request Request) {
	b, err := json.Marshal(request)
	crash(err)
	log.Print("Sending ", string(b))
	e := json.NewEncoder(s.socket)
	e.Encode(request)
}
